/**
******************************************************************************
* @file     main.c
* @author   AE team
* @version  V1.1.1
* @date     15/05/2019
* @brief
******************************************************************************
* @copy
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, MindMotion SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
* <h2><center>&copy; COPYRIGHT 2019 MindMotion</center></h2>
*/
#include "HAL_device.h"
#include "HAL_conf.h"
#include "stdio.h"

extern u32 SystemCoreClock;
void delay_init(void);
void delay_ms(__IO uint32_t nTime);
void TimingDelay_Decrement(void);
void LED_Init(void);
void KEY_Init(void);


static __IO uint32_t TimingDelay;


#define LED1_Port             GPIOB
#define LED1_Pin              GPIO_Pin_5
#define LED2_Port             GPIOB
#define LED2_Pin              GPIO_Pin_4
#define LED3_Port             GPIOB
#define LED3_Pin              GPIO_Pin_3
#define LED4_Port             GPIOA
#define LED4_Pin              GPIO_Pin_15

#define LED1_ON()             GPIO_ResetBits(LED1_Port,LED1_Pin)
#define LED1_OFF()            GPIO_SetBits(LED1_Port,LED1_Pin)
#define LED1_TOGGLE()         (GPIO_ReadOutputDataBit(LED1_Port,LED1_Pin))?(GPIO_ResetBits(LED1_Port,LED1_Pin)):(GPIO_SetBits(LED1_Port,LED1_Pin))



#define LED2_ON()             GPIO_ResetBits(LED2_Port,LED2_Pin)
#define LED2_OFF()            GPIO_SetBits(LED2_Port,LED2_Pin)
#define LED2_TOGGLE()         (GPIO_ReadOutputDataBit(LED2_Port,LED2_Pin))?(GPIO_ResetBits(LED2_Port,LED2_Pin)):(GPIO_SetBits(LED2_Port,LED2_Pin))


#define LED3_ON()             GPIO_ResetBits(LED3_Port,LED3_Pin)
#define LED3_OFF()            GPIO_SetBits(LED3_Port,LED3_Pin)
#define LED3_TOGGLE()         (GPIO_ReadOutputDataBit(LED3_Port,LED3_Pin))?(GPIO_ResetBits(LED3_Port,LED3_Pin)):(GPIO_SetBits(LED3_Port,LED3_Pin))


#define LED4_ON()             GPIO_ResetBits(LED4_Port,LED4_Pin)
#define LED4_OFF()            GPIO_SetBits(LED4_Port,LED4_Pin)
#define LED4_TOGGLE()         (GPIO_ReadOutputDataBit(LED4_Port,LED4_Pin))?(GPIO_ResetBits(LED4_Port,LED4_Pin)):(GPIO_SetBits(LED4_Port,LED4_Pin))


#define SW1_GPIO_Port         GPIOC
#define SW1_Pin               GPIO_Pin_13
#define SW2_GPIO_Port         GPIOA
#define SW2_Pin               GPIO_Pin_0
#define SW3_GPIO_Port         GPIOB
#define SW3_Pin               GPIO_Pin_10
#define SW4_GPIO_Port         GPIOB
#define SW4_Pin               GPIO_Pin_11

uint8_t KEY1State, KEY2State, KEY3State, KEY4State;

void keystatusInit(void)
{
    KEY1State = 0; KEY2State = 0; KEY3State = 0; KEY4State = 0;
}

GPIO_TypeDef* GPIO_PORT[] = {LED1_Port,
                             LED2_Port,
                             LED3_Port,
                             LED4_Port
                            };

const uint16_t GPIO_PIN[] = {LED1_Pin,
                             LED2_Pin,
                             LED3_Pin,
                             LED4_Pin
                            };
typedef enum {
    LED1,
    LED2,
    LED3,
    LED4
} Led_TypeDef;

void LEDx_TOGGLE(GPIO_TypeDef* GPIOx, uint16_t GPIO_pin)
{
    (GPIO_ReadOutputDataBit(GPIOx, GPIO_pin)) ? (GPIO_ResetBits(GPIOx, GPIO_pin)) : (GPIO_SetBits(GPIOx, GPIO_pin));
}

void LED_Toggle(Led_TypeDef Led)
{
    (GPIO_ReadOutputDataBit(GPIO_PORT[Led], GPIO_PIN[Led])) ? (GPIO_ResetBits(GPIO_PORT[Led], GPIO_PIN[Led])) : (GPIO_SetBits(GPIO_PORT[Led], GPIO_PIN[Led]));
}
static u8 KEY1DOWNRELEASE = 0;
u8 getKEY1downRelease(void)
{
    u8 result = 0;
    if(KEY1DOWNRELEASE == 0) {
        if  ( KEY1State == 1 ) {
            KEY1DOWNRELEASE = 1;
        }
    } else if(KEY1DOWNRELEASE == 1) {
        if  ( KEY1State == 0 ) {
            KEY1DOWNRELEASE = 2;
        }
    }
    if(KEY1DOWNRELEASE == 2) {
        result = 1;
        KEY1DOWNRELEASE = 0;
    }
    return result;
}
static u8 KEY2DOWNRELEASE = 0;
u8 getKEY2downRelease(void)
{
    u8 result = 0;
    if(KEY2DOWNRELEASE == 0) {
        if  ( KEY2State == 1 ) {
            KEY2DOWNRELEASE = 1;
        }
    } else if(KEY2DOWNRELEASE == 1) {
        if  ( KEY2State == 0 ) {
            KEY2DOWNRELEASE = 2;
        }
    }
    if(KEY2DOWNRELEASE == 2) {
        result = 1;
        KEY2DOWNRELEASE = 0;
    }
    return result;
}
static u8 KEY3DOWNRELEASE = 0;
u8 getKEY3downRelease(void)
{
    u8 result = 0;
    if(KEY3DOWNRELEASE == 0) {
        if  ( KEY3State == 1 ) {
            KEY3DOWNRELEASE = 1;
        }
    } else if(KEY3DOWNRELEASE == 1) {
        if  ( KEY3State == 0 ) {
            KEY3DOWNRELEASE = 2;
        }
    }
    if(KEY3DOWNRELEASE == 2) {
        result = 1;
        KEY3DOWNRELEASE = 0;
    }
    return result;
}
static u8 KEY4DOWNRELEASE = 0;
u8 getKEY4downRelease(void)
{
    u8 result = 0;
    if(KEY4DOWNRELEASE == 0) {
        if  ( KEY4State == 1 ) {
            KEY4DOWNRELEASE = 1;
        }
    } else if(KEY4DOWNRELEASE == 1) {
        if  ( KEY4State == 0 ) {
            KEY4DOWNRELEASE = 2;
        }
    }
    if(KEY4DOWNRELEASE == 2) {
        result = 1;
        KEY4DOWNRELEASE = 0;
    }
    return result;
}
void test(void)
{
    while (1) {
        if  ( getKEY1downRelease() == 1 ) {
            LED_Toggle(LED1);

        }
        if  ( getKEY2downRelease() == 1 ) {
            LED_Toggle(LED2);
        }
        if  ( getKEY3downRelease() == 1 ) {
            LED_Toggle(LED3);
        }
        if  ( getKEY4downRelease() == 1 ) {
            LED_Toggle(LED4);
        }
    }
}
static u16 KEY1BitBuf = 0x0;
static u16 KEY2BitBuf = 0x0;
static u16 KEY3BitBuf = 0x0;
static u16 KEY4BitBuf = 0x0;
void Key_Scan(void)
{
    /*SW1Ƿ */
    KEY1BitBuf =  ( KEY1BitBuf << 1 ) ;
    if ( GPIO_ReadInputDataBit(SW1_GPIO_Port, SW1_Pin) == Bit_RESET ) {
        KEY1BitBuf = KEY1BitBuf | 0x01; //1λǰֵλ
        if ( (0xFFFF) == KEY1BitBuf ) { //16ɨ,а룬16ڶ⵽״̬Ϊ
            //1MSжһΣsw1Countڵ32Ѱ32ms
            KEY1State = 1;                                                   //ð־
        }
    } else {
        if ( 0x0 == KEY1BitBuf ) { //
            KEY1State = 0;
        }
    }
    /*SW2Ƿ */
    KEY2BitBuf =  ( KEY2BitBuf << 1 ) ;
    if ( GPIO_ReadInputDataBit(SW2_GPIO_Port, SW2_Pin) ==  Bit_SET ) {
        KEY2BitBuf = KEY2BitBuf | 0x01; //1λǰֵλ
        if ( (0xFFFF) == KEY2BitBuf ) { //16ɨ,а룬16ڶ⵽״̬Ϊ
            //1MSжһΣsw1Countڵ32Ѱ32ms
            KEY2State = 1;                                                   //ð־
        }
    } else {
        if ( 0x0 == KEY2BitBuf ) { //
            KEY2State = 0;
        }
    }
    /*SW3Ƿ */
    KEY3BitBuf =  ( KEY3BitBuf << 1 ) ;
    if ( GPIO_ReadInputDataBit(SW3_GPIO_Port, SW3_Pin) == Bit_RESET ) {
        KEY3BitBuf = KEY3BitBuf | 0x01; //1λǰֵλ
        if ( (0xFFFF) == KEY3BitBuf ) { //16ɨ,а룬16ڶ⵽״̬Ϊ
            //1MSжһΣsw1Countڵ32Ѱ32ms
            KEY3State = 1;                                                   //ð־
        }
    } else {
        if ( 0x0 == KEY3BitBuf ) { //
            KEY3State = 0;
        }
    }
    /*SW4Ƿ */
    KEY4BitBuf =  ( KEY4BitBuf << 1 ) ;
    if ( GPIO_ReadInputDataBit(SW4_GPIO_Port, SW4_Pin) == Bit_RESET ) {
        KEY4BitBuf = KEY4BitBuf | 0x01; //1λǰֵλ
        if ( (0xFFFF) == KEY4BitBuf ) { //16ɨ,а룬16ڶ⵽״̬Ϊ
            //1MSжһΣsw1Countڵ32Ѱ32ms
            KEY4State = 1;                                                   //ð־
        }
    } else {
        if ( 0x0 == KEY4BitBuf ) { //
            KEY4State = 0;
        }
    }
}
/* USER CODE END 0 */
/********************************************************************************************************
**Ϣ main(void)
** 
** 
** 
********************************************************************************************************/
int main(void)
{
    delay_init();
    LED_Init();                                                                 //ʼLEDӵӲӿ
    KEY_Init();                                                                 //ʼ밴ӵӲӿ
    keystatusInit();
    test();
    while(1) {
    }
}

/********************************************************************************************************
**Ϣ delay_init(void)
** systickʱʼ
** 
** 
********************************************************************************************************/
void delay_init(void)
{
    if (SysTick_Config(SystemCoreClock / 1000)) {
        /* Capture error */
        while (1);
    }
    /* Configure the SysTick handler priority */
    NVIC_SetPriority(SysTick_IRQn, 0x0);                                        //SysTickжȼ
}

/********************************************************************************************************
**Ϣ SysTick_Handler(void)
** жϺSystickеݼ
** 
** 
********************************************************************************************************/
void SysTick_Handler(void)
{
    TimingDelay_Decrement();
    Key_Scan();
}

/********************************************************************************************************
**Ϣ TimingDelay_Decrement(void)
** 1msٶȵݼ
** pclk2ϵͳʱΪ8MHzӦ8
** 
********************************************************************************************************/
void TimingDelay_Decrement(void)
{
    if (TimingDelay != 0x00) {
        TimingDelay--;
    }
}

/********************************************************************************************************
**Ϣ delay_ms(__IO uint32_t nTime)
** Ӧõʱʹsystick
** nTimeʱ
** 
********************************************************************************************************/
void delay_ms(__IO uint32_t nTime)
{
    TimingDelay = nTime;

    while(TimingDelay != 0);
}

/*******************************************************************************
* @name   : GPIO_Clock_Set
* @brief  : RCC clock set
* @param  : Portx , State
* @retval : void
*******************************************************************************/
void GPIO_Clock_Set(GPIO_TypeDef* GPIOx, FunctionalState NewState)
{

    if(GPIOx == GPIOA) {
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, NewState);                   //GPIO clock starts
    }
    if(GPIOx == GPIOB) {
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, NewState);                   //GPIO clock starts
    }
    if(GPIOx == GPIOC) {
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, NewState);                   //GPIO clock starts
    }
    if(GPIOx == GPIOD) {
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOD, NewState);                   //GPIO clock starts
    }
}
/********************************************************************************************************
**Ϣ LED_Init(void)
** LEDʼ
** 
** 
********************************************************************************************************/
void LED_Init(void)
{

    GPIO_InitTypeDef  GPIO_InitStructure;

    GPIO_Clock_Set(GPIOA, ENABLE);  //GPIOAʱ
    GPIO_Clock_Set(GPIOB, ENABLE);  //GPIOBʱ
    GPIO_InitStructure.GPIO_Pin  =  GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    LED1_ON();
    LED2_ON();
    LED3_ON();
    LED4_ON();
}

/*******************************************************************************
**Ϣ void KEY_Init(void)
** ʼ
** 
** 
**    ע 
*******************************************************************************/
void KEY_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;


    GPIO_Clock_Set(GPIOA, ENABLE);  //GPIOAʱ
    GPIO_Clock_Set(GPIOB, ENABLE);  //GPIOBʱ
    GPIO_Clock_Set(GPIOC, ENABLE);  //GPIOCʱ
    GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_13;                                 //PC13K1
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;                               //ó
    GPIO_Init(GPIOC, &GPIO_InitStructure);                                      //ʼGPIOC13

    GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_0;                                  //PA0,K2WK_UP
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;                               //ó
    GPIO_Init(GPIOA, &GPIO_InitStructure);                                      //ʼGPIOA0

    GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_10 | GPIO_Pin_11;                   //PB10,PB11,K3,K4
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;                               //ó
    GPIO_Init(GPIOB, &GPIO_InitStructure);                                      //ʼGPIOB.10,11
}


/**
* @}
*/

/**
* @}
*/

/**
* @}
*/

/*-------------------------(C) COPYRIGHT 2019 MindMotion ----------------------*/

