/**
******************************************************************************
* @file     main.c
* @author   AE team
* @version  V1.1.1
* @date     15/05/2019
* @brief
******************************************************************************
* @copy
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, MindMotion SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
* <h2><center>&copy; COPYRIGHT 2019 MindMotion</center></h2>
*/
#include "HAL_device.h"
#include "HAL_conf.h"
#include "stdio.h"

char printBuf[100];
extern u32 SystemCoreClock;
void delay_init(void);
void delay_ms(__IO uint32_t nTime);
void TimingDelay_Decrement(void);
void LED_Init(void);
void uart_initwBaudRate(u32 bound);
void UartSendGroup(u8* buf, u16 len);
void WKUP_Init(void);
u8 Check_WKUP(void) ;
void Sys_Standby(void);

static __IO uint32_t TimingDelay;

#define LED4_ON()  GPIO_ResetBits(GPIOA,GPIO_Pin_15)	// PA15
#define LED4_OFF()  GPIO_SetBits(GPIOA,GPIO_Pin_15)	// PA15
#define LED4_TOGGLE()  (GPIO_ReadOutputDataBit(GPIOA,GPIO_Pin_15))?(GPIO_ResetBits(GPIOA,GPIO_Pin_15)):(GPIO_SetBits(GPIOA,GPIO_Pin_15)) // PA15

#define LED3_ON()  GPIO_ResetBits(GPIOB,GPIO_Pin_3)	// PB3
#define LED3_OFF()  GPIO_SetBits(GPIOB,GPIO_Pin_3)	// PB3
#define LED3_TOGGLE()  (GPIO_ReadOutputDataBit(GPIOB,GPIO_Pin_3))?(GPIO_ResetBits(GPIOB,GPIO_Pin_3)):(GPIO_SetBits(GPIOB,GPIO_Pin_3))	// PB3

#define LED2_ON()  GPIO_ResetBits(GPIOB,GPIO_Pin_4)	// PB4
#define LED2_OFF()  GPIO_SetBits(GPIOB,GPIO_Pin_4)	// PB4
#define LED2_TOGGLE()  (GPIO_ReadOutputDataBit(GPIOB,GPIO_Pin_4))?(GPIO_ResetBits(GPIOB,GPIO_Pin_4)):(GPIO_SetBits(GPIOB,GPIO_Pin_4))	// PB4

#define LED1_ON()  GPIO_ResetBits(GPIOB,GPIO_Pin_5)	// PB5
#define LED1_OFF()  GPIO_SetBits(GPIOB,GPIO_Pin_5)	// PB5
#define LED1_TOGGLE()  (GPIO_ReadOutputDataBit(GPIOB,GPIO_Pin_5))?(GPIO_ResetBits(GPIOB,GPIO_Pin_5)):(GPIO_SetBits(GPIOB,GPIO_Pin_5))	// PB5

/********************************************************************************************************
**Ϣ main(void)
** 
** 
** 
********************************************************************************************************/
int main(void)
{
    delay_init();
    LED_Init();                                                                 //ʼLEDӵӲӿ
    uart_initwBaudRate(115200);
    UartSendGroup((u8*)printBuf, sprintf(printBuf, "밴wake up3벻\r\n"));
    WKUP_Init();                                                                //ʼWK_UPͬʱǷ

    while(1) {
        LED1_TOGGLE();
        LED2_TOGGLE();
        LED3_TOGGLE();
        LED4_TOGGLE();
        delay_ms(250);
        UartSendGroup((u8*)printBuf, sprintf(printBuf, "Ѵstandbyģʽ\r\n"));
    }

}

/********************************************************************************************************
**Ϣ delay_init(void)
** systickʱʼ
** 
** 
********************************************************************************************************/
void delay_init(void)
{
    if (SysTick_Config(SystemCoreClock / 1000)) {
        /* Capture error */
        while (1);
    }
    /* Configure the SysTick handler priority */
    NVIC_SetPriority(SysTick_IRQn, 0x0);                                        //SysTickжȼ
}

/********************************************************************************************************
**Ϣ SysTick_Handler(void)
** жϺSystickеݼ
** 
** 
********************************************************************************************************/
void SysTick_Handler(void)
{
    TimingDelay_Decrement();
}

/********************************************************************************************************
**Ϣ TimingDelay_Decrement(void)
** 1msٶȵݼ
** pclk2ϵͳʱΪ8MHzӦ8
** 
********************************************************************************************************/
void TimingDelay_Decrement(void)
{
    if (TimingDelay != 0x00) {
        TimingDelay--;
    }
}

/********************************************************************************************************
**Ϣ delay_ms(__IO uint32_t nTime)
** Ӧõʱʹsystick
** nTimeʱ
** 
********************************************************************************************************/
void delay_ms(__IO uint32_t nTime)
{
    TimingDelay = nTime;

    while(TimingDelay != 0);
}

/********************************************************************************************************
**Ϣ LED_Init(void)
** LEDʼ
** 
** 
********************************************************************************************************/
void LED_Init(void)
{

    GPIO_InitTypeDef  GPIO_InitStructure;

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);   //GPIOA,GPIOBʱ

    GPIO_InitStructure.GPIO_Pin  =  GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    LED1_ON();
    LED2_ON();
    LED3_ON();
    LED4_ON();
}

/********************************************************************************************************
**Ϣ uart_initwBaudRate(u32 bound)
** UARTʼ
** bound
** 
********************************************************************************************************/
void uart_initwBaudRate(u32 bound)
{
    //GPIO˿
    GPIO_InitTypeDef GPIO_InitStructure;
    UART_InitTypeDef UART_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_UART1, ENABLE);                       //ʹUART1ʱ
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);                         //GPIOAʱ
    //UART ʼ
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_1);

    UART_InitStructure.UART_BaudRate = bound;                                   //ڲ
    UART_InitStructure.UART_WordLength = UART_WordLength_8b;                    //ֳΪ8λݸʽ
    UART_InitStructure.UART_StopBits = UART_StopBits_1;                         //һֹͣλ
    UART_InitStructure.UART_Parity = UART_Parity_No;                            //żУλ
    UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None;//Ӳ
    UART_InitStructure.UART_Mode = UART_Mode_Rx | UART_Mode_Tx;	                //շģʽ

    UART_Init(UART1, &UART_InitStructure);                                      //ʼ1
    UART_Cmd(UART1, ENABLE);                                                    //ʹܴ1

    //UART1_TX   GPIOA.9
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;                                   //PA.9
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;                             //
    GPIO_Init(GPIOA, &GPIO_InitStructure);                                      //ʼGPIOA.9

    //UART1_RX	  GPIOA.10ʼ
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;                                  //PA10
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;                       //
    GPIO_Init(GPIOA, &GPIO_InitStructure);                                      //ʼGPIOA.10

}

/********************************************************************************************************
**Ϣ WKUP_Init(void)
** ⲿжϻѴʼ
** 
** 
********************************************************************************************************/
void WKUP_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;
    EXTI_InitTypeDef EXTI_InitStructure;

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);                      //ʹGPIOA͸ùʱ

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;	                               //PA.0
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;                               //
    GPIO_Init(GPIOA, &GPIO_InitStructure);                                      //ʼIO
    //ʹⲿжϷʽ
    SYSCFG_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);               //ж0GPIOA.0

    EXTI_InitStructure.EXTI_Line = EXTI_Line0;	                                //ðеⲿ·
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;		 	                //ⲿжģʽ:EXTI·Ϊж
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;                      //ش
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);                                             // ʼⲿж

    NVIC_InitStructure.NVIC_IRQChannel = EXTI0_1_IRQn;                          //ʹܰڵⲿжͨ
    NVIC_InitStructure.NVIC_IRQChannelPriority = 2;                             //ȼ2
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;                             //ʹⲿжͨ
    NVIC_Init(&NVIC_InitStructure);                                             //NVIC_InitStructָĲʼNVICĴ

    if(Check_WKUP() == 0) Sys_Standby();                                        //ǿ,ģʽ
}

/********************************************************************************************************
**Ϣ Check_WKUP(void)
** жϰǷ
** 
** 
********************************************************************************************************/
u8 Check_WKUP(void)
{
    u8 t = 0;	//¼µʱ
    LED1_ON();
    LED2_ON();
    LED3_ON();
    LED4_ON();

    while(1) {
        if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0)) {
            t++;
            delay_ms(20);
            if(t >= 100) {                                                      //³2
                LED1_ON();
                LED2_ON();
                LED3_ON();
                LED4_ON();
                return 1;                                                       //3s
            }

        } else {
            LED1_OFF();
            LED2_OFF();
            LED3_OFF();
            LED4_OFF();
            return 0;                                                           //²3
        }

    }

}

/********************************************************************************************************
**Ϣ EXTI0_IRQHandler(void)
** ⲿж0жϱ־λ
** 
** 
********************************************************************************************************/
void EXTI0_1_IRQHandler(void)
{
    EXTI_ClearITPendingBit(EXTI_Line0);                                         // LINE10ϵжϱ־λ
}

/********************************************************************************************************
**Ϣ Sys_Standby(void)
** ģʽ
** 
** 
********************************************************************************************************/
void Sys_Standby(void)
{
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);                         //ʹPWRʱ
    //	RCC->APB2RSTR|=0X01FC;//λIO
    PWR_WakeUpPinCmd(ENABLE);                                                   //ʹܻѹܽŹ
    PWR_EnterSTANDBYMode();                                                     //STANDBYģʽ

}

/********************************************************************************************************
**Ϣ void UartSendByte(u8 dat)
** UART
** dat
** 
**    ע 
********************************************************************************************************/
void UartSendByte(u8 dat)
{
    UART_SendData(UART1, dat);
    while(!UART_GetFlagStatus(UART1, UART_FLAG_TXEPT));
}

/********************************************************************************************************
**Ϣ void UartSendGroup(u8* buf,u16 len)
** UART
** buf,len
** 
**    ע 
********************************************************************************************************/
void UartSendGroup(u8* buf, u16 len)
{
    while(len--)
        UartSendByte(*buf++);
}

/**
* @}
*/

/**
* @}
*/

/**
* @}
*/

/*-------------------------(C) COPYRIGHT 2019 MindMotion ----------------------*/

